﻿using System;
using System.Xml;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.IO;
using System.Windows.Shapes;


namespace XamlTester
{

    public class DataReaderWriter 
    {
        #region Variables and constants

        // STRING PORTIONS FOR XML
        const string SLASH = "/";
        const string CODES = "Codes";

        // Paths to standard elements, always included but not always made visible
        const string FILE = "_File";
        const string FILEPATH = CODES + SLASH + FILE;

        const string FOLDER = "_Folder";
        const string FOLDERPATH = CODES + SLASH + FOLDER;

        const string DATE = "_Date";
        const string DATEPATH = CODES + SLASH + DATE; 
        
        const string TIME = "_Time";
        const string TIMEPATH = CODES + SLASH + TIME;

        // These are generic to all codes that want to use them
        const string DATA = "Data";             // the data describing the attributes of that code
        const string DEFAULT = "DefaultValue";  // a default value for that code
        const string LIST = "List";             // indicates a list of items
        const string ITEM = "Item";             // and item in a list
        const string LABEL = "Label";           // a label used to describe that code
        const string VISIBLE = "Visible";        // whether an item should be visible (used by standard items)
        const string TXTBOXWIDTH = "TXTBOXWIDTH";      // the width of the textbox

        // Path to Notes

        const string NOTE = "Note";
        const string NOTEPATH = CODES + SLASH + NOTE;
 
        // Path to Counters
        const string COUNTER = "Counter";
        const string COUNTERPATH = CODES + SLASH + COUNTER;

        // Path to Fixed Choices
        const string FIXEDCHOICES = "FixedChoices";
        const string FIXEDCHOICE = "FixedChoice";
        const string FIXEDCHOICEPATH = CODES + SLASH + FIXEDCHOICE;

        // Constants For setting defaults for control parameters
        Thickness THICKNESS_INNER = new Thickness (0,3,0,3);  // The space between the sets of widget
        Thickness THICKNESS_INNER_RADIOBUTTON = new Thickness(0, 5, 3, 0);  // The space between the sets of widget
        Thickness THICKNESS_COUNTER_CONTENTS = new Thickness(5, 3, 15, 3);
        Thickness THICKNESS_OUTER = new Thickness (0,0,15,0);  // The space between widgets within a set
        Thickness THICK_LEFT = new Thickness(5, 0, 0, 0);
        Thickness THICK_RIGHT = new Thickness(0, 0, 5, 0);
        Thickness THICK_LEFTRIGHT = new Thickness(5, 0, 2, 0); 

        const int TXTBOXWIDTHDEFAULT = 100;
        const int CTRWIDTHDEFAULT = 30;
        #endregion

        #region Constructors and Destructors
        // Constructor
        public  DataReaderWriter()
        {
        }
        #endregion

        #region Read the Codes
        public void ReadCodes(string filePath, Window mainWindow, ToolBarTray toolbarTray)
        {

            XmlNodeList nodelist;
            XmlDocument xmlDoc = new XmlDocument();
            bool visibilityState = false;
            xmlDoc.Load(filePath);

            // Create and add the Toolbars to the tray
            ToolbarUI tbc = new ToolbarUI();
            tbc.toolbarTray = toolbarTray;
            tbc.AddToolbarsToTray();

            // Add Filename to the Standards Toolbar
            nodelist = xmlDoc.SelectNodes(FILEPATH);
            if (nodelist.Count == 1) // There should be only one
            {
                XmlNodeList nodeData = nodelist[0].SelectNodes(DATA);
                Note fileNote = new Note(tbc.toolbarStandard);
                fileNote.Label = TextFromNode(nodeData, 0, LABEL);
                fileNote.Default = TextFromNode(nodeData, 0, DEFAULT);
                fileNote.Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                fileNote.ReadOnly = true;
                visibilityState = (TextFromNode(nodeData, 0, VISIBLE) == "true") ? true : false;
                fileNote.Configure(visibilityState);
            }

            // Add Date to the Standards Toolbar
            nodelist = xmlDoc.SelectNodes(DATEPATH);
            if (nodelist.Count == 1) // There should be only one
            {
                XmlNodeList nodeData = nodelist[0].SelectNodes(DATA);
                Note dateNote = new Note(tbc.toolbarStandard);
                dateNote.Label = TextFromNode(nodeData, 0, LABEL);
                dateNote.Default = TextFromNode(nodeData, 0, DEFAULT);
                dateNote.Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                visibilityState = (TextFromNode(nodeData, 0, VISIBLE) == "true") ? true : false;
                dateNote.Configure(visibilityState);
            }

            // Add Time to the Standards Toolbar
            nodelist = xmlDoc.SelectNodes(TIMEPATH);
            if (nodelist.Count == 1) // There should be only one
            {
                XmlNodeList nodeData = nodelist[0].SelectNodes(DATA);
                Note timeNote = new Note(tbc.toolbarStandard);
                timeNote.Label = TextFromNode(nodeData, 0, LABEL);
                timeNote.Default = TextFromNode(nodeData, 0, DEFAULT);
                timeNote.Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                visibilityState = (TextFromNode(nodeData, 0, VISIBLE) == "true") ? true : false;
                timeNote.Configure(visibilityState);
            }

            // Add Folder to the Standards Toolbar
            nodelist = xmlDoc.SelectNodes(FOLDERPATH);
            if (nodelist.Count == 1) // There should be only one
            {
                XmlNodeList nodeData = nodelist[0].SelectNodes(DATA);
                Note folderNote = new Note(tbc.toolbarStandard);
                folderNote.Label = TextFromNode(nodeData, 0, LABEL);
                folderNote.Default = TextFromNode(nodeData, 0, DEFAULT);
                folderNote.Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                folderNote.ReadOnly = true;
                visibilityState = (TextFromNode(nodeData, 0, VISIBLE) == "true") ? true : false;
                folderNote.Configure(visibilityState);
            }

             // Add Counters to the Counters toolbar
            nodelist = xmlDoc.SelectNodes(COUNTERPATH);
            Counter[] counter = new Counter[nodelist.Count]; 
            for (int i = 0; i < nodelist.Count; i++)
            {
                // Get the XML section containing values for each note
                XmlNodeList nodeData = nodelist[i].SelectNodes(DATA);


                //create and configure each counter
                counter[i] = new Counter(mainWindow, tbc.toolbarCounters);
                counter[i].Label = TextFromNode(nodeData, 0, LABEL);
                counter[i].Default = TextFromNode(nodeData, 0, DEFAULT);
                counter[i].Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                counter[i].Configure(true);
            }

            // Add Notes to the Notes toolbar
            nodelist = xmlDoc.SelectNodes(NOTEPATH);
            Note[] note = new Note[nodelist.Count];
            for (int i = 0; i < nodelist.Count; i++)
            {
                // Get the XML section containing values for each note
                XmlNodeList nodeData = nodelist[i].SelectNodes(DATA);

                //create and configure each note
                note[i] = new Note(tbc.toolbarNotes);
                note[i].Default = TextFromNode(nodeData, 0, DEFAULT);
                note[i].Label = TextFromNode(nodeData, 0, LABEL);
                note[i].Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                note[i].Configure(true);
            }

            // Add FixedChoices to the Fixed Choices toolbar                
            nodelist = xmlDoc.SelectNodes(FIXEDCHOICEPATH);
             
            FixedChoice [] fixedChoice = new FixedChoice[nodelist.Count];
            for (int i = 0; i < nodelist.Count; i++)
            {
                // Get the XML section containing values for each note
                XmlNodeList nodeData = nodelist[i].SelectNodes(DATA);

                //create and configure each counter
                fixedChoice[i] = new FixedChoice(tbc.toolbarFixedChoices);
                fixedChoice[i].Default = TextFromNode(nodeData, 0, DEFAULT);
                fixedChoice[i].Label = TextFromNode(nodeData, 0, LABEL);
                fixedChoice[i].Width = TextFromNode(nodeData, 0, TXTBOXWIDTH);
                // Get the items in each counter
                XmlNodeList nItems = nodelist[i].SelectNodes(DATA + SLASH + LIST + SLASH + ITEM);
                string[] list = new string[nItems.Count];
                int idx = 0; 
                foreach (XmlNode nodeItem in nItems)
                {
                    list[idx++] = nodeItem.InnerText;
                }
                fixedChoice[i].Items = list;
                fixedChoice[i].Configure(true);
            }
         }
        #endregion

        #region Utilities
        // Given a nodelist, get the text associated with it 
        private string TextFromNode(XmlNodeList node, int nodeIndex, string nodeToFind)
        {
            XmlNodeList n = node[nodeIndex].SelectNodes(nodeToFind);
            return n[0].InnerText;
        }
        #endregion
    }
       
}
