﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;

namespace XamlTester
{
    #region Class ToolbarUI - set up and configure the toolbars
    class ToolbarUI
    {

        // Properties
        public ToolBarTray toolbarTray { get; set; }  // The toolbar tray that holds all the toolbars
        public ToolBar toolbarStandard { get; set; }
        public ToolBar toolbarCounters { get; set; }
        public ToolBar toolbarNotes { get; set; }
        public ToolBar toolbarFixedChoices { get; set; }

        // Constructors 
        public ToolbarUI() 
        {
            this.toolbarStandard = new ToolBar();
            this.toolbarCounters = new ToolBar();
            this.toolbarNotes = new ToolBar();
            this.toolbarFixedChoices = new ToolBar();
        }

        // Add all the toolbars to the tray, placing them in particular bands
        public void AddToolbarsToTray()
        {
            // First put the toolbars into the correct bands
            this.toolbarStandard.Band = 1;
            this.toolbarCounters.Band = 2;
            this.toolbarFixedChoices.Band = 3;
            this.toolbarNotes.Band = 4;

            // and now add them to the toolbar tray
            this.toolbarTray.ToolBars.Add(this.toolbarStandard);
            this.toolbarTray.ToolBars.Add(this.toolbarCounters);
            this.toolbarTray.ToolBars.Add(this.toolbarFixedChoices);
            this.toolbarTray.ToolBars.Add(this.toolbarNotes);
        }
    }
    #endregion

    #region Note
    // A note comprises 
    // - a label containing the descriptive label) 
    // - an editable textbox (containing the content) at the given width
    class Note
    {
        private ToolBar toolbar;
        private Label labelCtl;
        private TextBox contentCtl;

        const int DEFAULTWIDTH = 100;  // The default width of content controls
        Thickness THICK_LEFT = new Thickness(5, 0, 0, 0);
        Thickness THICK_RIGHT = new Thickness(0, 0, 5, 0);
        Thickness THICK_LEFTRIGHT = new Thickness(5, 0, 2, 0); 

        // Constructor
        // Initialize each control
        public Note (ToolBar tb) 
        {
            this.labelCtl = new Label();
            this.contentCtl = new TextBox();
            this.toolbar = tb;
        }

        public string Label  {  set  {labelCtl.Content = value;  } }
        public string Default { set { contentCtl.Text = value; } }
        public string Width {
            set 
            {
                int w;
                this.contentCtl.Width = (Int32.TryParse(value, out w)) ? w : DEFAULTWIDTH; 
            } 
        }
        public bool ReadOnly {
            set {this.contentCtl.IsEnabled = !value;}
        }
 

        public void Configure(bool visible)
        {
            //This label will become the text label
            this.labelCtl.FontWeight = FontWeights.Bold;
            this.labelCtl.Height = 28;
            this.labelCtl.Padding = THICK_LEFTRIGHT;
            this.labelCtl.VerticalContentAlignment = VerticalAlignment.Center;

            this.contentCtl.BorderThickness = new Thickness(1);
            this.contentCtl.BorderBrush = Brushes.LightBlue;
            this.contentCtl.VerticalContentAlignment = VerticalAlignment.Center;
            this.contentCtl.Margin = THICK_RIGHT;

            if (!visible) return; 
            this.toolbar.Items.Add(labelCtl);
            this.toolbar.Items.Add(contentCtl);
        }
    }
    #endregion

    #region Counter
    // A counter comprises 
    // - a radiobutton (which also contains a label that becomes a toggle)
    // - a non-editable textbox containing the content at the given width
    class Counter
    {
        private ToolBar toolbar; 
        private TextBox contentCtl;
        private RadioButton labelCtl;
        //private Brush outlineBrush; 

        const int DEFAULTWIDTH = 100;  // The default width of content controls
        Thickness THICK_LEFT = new Thickness(5, 0, 0, 0);
        Thickness THICK_RIGHT = new Thickness(0, 0, 5, 0);
        Thickness THICK_LEFTRIGHT = new Thickness(5, 0, 2, 0);

        // Constructor
        // Initialize each control
        public Counter(Window mainWindow, ToolBar tb)
        {
            this.labelCtl = new RadioButton();
            this.contentCtl = new TextBox();
            this.toolbar = tb;
        }

        public string Label { set { labelCtl.Content = value; } }
        public string Default { set { contentCtl.Text = value; } }
        public string Width
        {
            set
            {
                int w;
                this.contentCtl.Width = (Int32.TryParse(value, out w)) ? w : DEFAULTWIDTH;
            }
        }

        public void Configure(bool visible)
        {
            //This label will become the text label
            this.labelCtl.GroupName = "A";
            this.labelCtl.FontWeight = FontWeights.Bold;
            this.labelCtl.Height = 28;
            this.labelCtl.Padding = THICK_LEFTRIGHT;
            this.labelCtl.VerticalContentAlignment = VerticalAlignment.Center;

            this.contentCtl.BorderThickness = new Thickness(1);
            this.contentCtl.Height = this.labelCtl.Height;
            this.contentCtl.BorderBrush = Brushes.LightBlue;
            this.contentCtl.VerticalContentAlignment = VerticalAlignment.Center;
            this.contentCtl.Margin = THICK_RIGHT;

            if (!visible) return;
            this.toolbar.Items.Add(this.labelCtl);
            this.toolbar.Items.Add(this.contentCtl);
        }
     }
    #endregion

    #region FixedChoice
    // A Fixed comprises 
    // - a label containing the descriptive label) 
    // - an editable combobox menu
    class FixedChoice
    {
        private ToolBar toolbar;
        private Label labelCtl;
        private ComboBox contentCtl;

        const int DEFAULTWIDTH = 100;  // The default width of content controls
        Thickness THICK_LEFT = new Thickness(5, 0, 0, 0);
        Thickness THICK_RIGHT = new Thickness(0, 0, 5, 0);
        Thickness THICK_LEFTRIGHT = new Thickness(5, 0, 2, 0);

        // Constructor
        // Initialize each control
        public FixedChoice(ToolBar tb)
        {
            this.labelCtl = new Label();
            this.contentCtl = new ComboBox();
            this.toolbar = tb;
        }

        public string Label { set { labelCtl.Content = value; } }
        public string Default { set { contentCtl.Text = value; } }
        public string Width
        {
            set
            {
                int w;
                this.contentCtl.Width = (Int32.TryParse(value, out w)) ? w : DEFAULTWIDTH;
            }
        }

        public string[] Items 
        {
            set 
            {
                foreach (string str in value)
                {
                    ComboBoxItem cbi = new ComboBoxItem();
                    cbi.Content = str;
                    this.contentCtl.Items.Add(cbi.Content);
                }
            }
        }

        public void Configure(bool visible)
        {
            //This label will become the text label
            this.labelCtl.FontWeight = FontWeights.Bold;
            this.labelCtl.Height = 28;
            this.labelCtl.Padding = THICK_LEFTRIGHT;
            this.labelCtl.VerticalContentAlignment = VerticalAlignment.Center;
            this.contentCtl.IsEditable = true; 
            this.contentCtl.BorderThickness = new Thickness(1);
            this.contentCtl.BorderBrush = Brushes.LightBlue;
            this.contentCtl.VerticalContentAlignment = VerticalAlignment.Center;
            this.contentCtl.Margin = THICK_RIGHT;

            if (!visible) return;
            this.toolbar.Items.Add(labelCtl);
            this.toolbar.Items.Add(contentCtl);
        }
    }
    #endregion
}
